header ="""
import sys

from pyamlside2.mainwindow import PyamlSide2Window
from PySide2.QtWidgets import QApplication

class MainWindow(PyamlSide2Window):
    def __init__(self, args):
        self.number = 0
        if (len(args) == 2):
            super().__init__(args[1])
        else:
            print("No args.")
            exit(1)

"""

middle ="""
        self.show()

"""

footer ="""
def entry_point():
    app = QApplication(sys.argv)
    window = MainWindow(sys.argv)
    sys.exit(app.exec_())

if __name__ == '__main__':
    entry_point()
"""

import os
from pyamlside2.label_configure import label_configure
from pyamlside2.utils.get_yaml import get_yaml
from pyamlside2.drawio_parse.parse import xml2yaml

import argparse

class generate_template():
    def __init__(self):
        self.tab = "    "
        self.tab2 = "        "

        self.widgets_keyword = {
            "qpushbutton": "clicked",
            "qlineedit": "textChanged",
            "qcheckbox": "stateChanged",
            "qcombobox": "currentIndexChanged",
            "qspinbox": "valueChanged",
            "qslider": "valueChanged",
            "qprogressbar": "valueChanged",
        }

    def generate_py(self, input_yaml_path:str) -> str:
        yaml_abs_path = os.path.abspath(input_yaml_path)

        # add definition
        export_py_str = header
        yaml_data = get_yaml(yaml_abs_path)
        for key in yaml_data:
            config = label_configure(yaml_abs_path, key)
            # add function
            for widget_key in self.widgets_keyword:
                if widget_key in config.type:
                    export_py_str += self.tab2 + "self.widgets[\"" + key + "\"]." + self.widgets_keyword[widget_key] + ".connect(self." + key + "_update" + ")\n"

        export_py_str += middle

        for key in yaml_data:
            config = label_configure(yaml_abs_path, key)
            # add function
            for widget_key in self.widgets_keyword:
                if widget_key in config.type:
                    export_py_str += self.tab + "def " + key + "_update(self):\n"
                    export_py_str += self.tab2 + "print(\"" + key + " clicked.\")\n"
                    # print slider value
                    if widget_key == "qslider":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].value())\n"
                    # print combobox value
                    if widget_key == "qcombobox":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].currentText())\n"
                    # print checkbox value
                    if widget_key == "qcheckbox":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].isChecked())\n"
                    # print lineedit value
                    if widget_key == "qlineedit":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].text())\n"
                    # print spinbox value
                    if widget_key == "qspinbox":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].value())\n"
                    # print progressbar value
                    if widget_key == "qprogressbar":
                        export_py_str += self.tab2 + "print(\"" + key + " value: \", self.widgets[\"" + key + "\"].value())\n"

                    export_py_str += "\n"
                    break

        export_py_str += footer
        return export_py_str

def entry_point():
    parser = argparse.ArgumentParser(description="Generate template.")
    parser.add_argument("--input", "-i", help="input xml file", required=True)
    parser.add_argument("--output-dir", "-o", help="output directory", required=True)
    args = parser.parse_args()

    print("================== rw_gen_xml2all ======================")
    print("input: " + args.input)
    print("output-dir: " + args.output_dir)
    print("=========================================================")

    os.makedirs(args.output_dir, exist_ok=True)
    intput_basename_no_exps = os.path.basename(args.input).split(".")[0]

    # generate yaml
    yaml_str = xml2yaml(args.input)
    with open(args.output_dir + "/" + intput_basename_no_exps + ".yaml", "w") as f:
        f.write(yaml_str)

    # generate py
    gen_py = generate_template()
    str_python = gen_py.generate_py(args.output_dir + "/" + intput_basename_no_exps + ".yaml")
    with open(args.output_dir + "/" + intput_basename_no_exps + ".py", "w") as f:
        f.write(str_python)

    print("Generate template done.")
    print("run `$ python3 " + args.output_dir + "/" + intput_basename_no_exps + ".py " + args.output_dir + "/" + intput_basename_no_exps + ".yaml`")

if __name__ == '__main__':
    entry_point()
